@@ -386,6 +386,11 @@ class Simulator {
   inline double ReadD(int64_t addr, Instruction* instr);
   inline void WriteD(int64_t addr, double value, Instruction* instr);
 
+  template <typename T>
+  T ReadMem(int64_t addr, Instruction* instr);
+  template <typename T>
+  void WriteMem(int64_t addr, T value, Instruction* instr);
+
   // Helper for debugging memory access.
   inline void DieOrDebug();
 
@@ -396,6 +401,10 @@ class Simulator {
   void TraceMSARegWr(T* value);
   void TraceMemWr(int64_t addr, int64_t value, TraceType t);
   void TraceMemRd(int64_t addr, int64_t value, TraceType t = DWORD);
+  template <typename T>
+  void TraceMemRd(int64_t addr, T value);
+  template <typename T>
+  void TraceMemWr(int64_t addr, T value);
 
   // Operations depending on endianness.
   // Get Double Higher / Lower word.
@@ -441,6 +450,8 @@ class Simulator {
   void DecodeTypeMsa2RF();
   template <typename T>
   T MsaI5InstrHelper(uint32_t opcode, T ws, int32_t i5);
+  template <typename T>
+  T MsaBitInstrHelper(uint32_t opcode, T wd, T ws, int32_t m);
 
   // Executing is handled based on the instruction type.
   void DecodeTypeRegister();
