@@ -1,11 +1,30 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 const common = require('../common');
-const assert = require('assert');
-
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
+
+const assert = require('assert');
 const crypto = require('crypto');
 
 crypto.DEFAULT_ENCODING = 'buffer';
@@ -307,141 +326,139 @@ const TEST_CASES = [
     tag: 'a44a8266ee1c8eb0c8b5d4cf5ae9f19a', tampered: false },
 ];
 
+const errMessages = {
+  auth: / auth/,
+  state: / state/,
+  FIPS: /not supported in FIPS mode/,
+  length: /Invalid IV length/,
+};
+
 const ciphers = crypto.getCiphers();
 
-for (const i in TEST_CASES) {
-  const test = TEST_CASES[i];
+common.expectWarning('Warning', (common.hasFipsCrypto ? [] : [
+  'Use Cipheriv for counter mode of aes-192-gcm'
+]).concat(
+  [0, 1, 2, 6, 9, 10, 11, 17]
+  .map((i) => `Permitting authentication tag lengths of ${i} bytes is ` +
+            'deprecated. Valid GCM tag lengths are 4, 8, 12, 13, 14, 15, 16.')
+));
 
-  if (ciphers.indexOf(test.algo) === -1) {
-    common.skip('unsupported ' + test.algo + ' test');
+for (const test of TEST_CASES) {
+  if (!ciphers.includes(test.algo)) {
+    common.printSkipMessage(`unsupported ${test.algo} test`);
     continue;
   }
 
   if (common.hasFipsCrypto && test.iv.length < 24) {
-    console.log('1..0 # Skipped: IV len < 12 bytes unsupported in FIPS mode');
+    common.printSkipMessage('IV len < 12 bytes unsupported in FIPS mode');
     continue;
   }
 
-  (function() {
-    var encrypt = crypto.createCipheriv(test.algo,
-      new Buffer(test.key, 'hex'), new Buffer(test.iv, 'hex'));
+  {
+    const encrypt = crypto.createCipheriv(test.algo,
+                                          Buffer.from(test.key, 'hex'),
+                                          Buffer.from(test.iv, 'hex'));
     if (test.aad)
-      encrypt.setAAD(new Buffer(test.aad, 'hex'));
+      encrypt.setAAD(Buffer.from(test.aad, 'hex'));
 
-    var inputEncoding = test.plainIsHex ? 'hex' : 'ascii';
-    var hex = encrypt.update(test.plain, inputEncoding, 'hex');
+    const inputEncoding = test.plainIsHex ? 'hex' : 'ascii';
+    let hex = encrypt.update(test.plain, inputEncoding, 'hex');
     hex += encrypt.final('hex');
 
-    var auth_tag = encrypt.getAuthTag();
+    const auth_tag = encrypt.getAuthTag();
     // only test basic encryption run if output is marked as tampered.
     if (!test.tampered) {
-      assert.equal(hex, test.ct);
-      assert.equal(auth_tag.toString('hex'), test.tag);
+      assert.strictEqual(hex, test.ct);
+      assert.strictEqual(auth_tag.toString('hex'), test.tag);
     }
-  })();
+  }
 
-  (function() {
-    var decrypt = crypto.createDecipheriv(test.algo,
-      new Buffer(test.key, 'hex'), new Buffer(test.iv, 'hex'));
-    decrypt.setAuthTag(new Buffer(test.tag, 'hex'));
+  {
+    const decrypt = crypto.createDecipheriv(test.algo,
+                                            Buffer.from(test.key, 'hex'),
+                                            Buffer.from(test.iv, 'hex'));
+    decrypt.setAuthTag(Buffer.from(test.tag, 'hex'));
     if (test.aad)
-      decrypt.setAAD(new Buffer(test.aad, 'hex'));
+      decrypt.setAAD(Buffer.from(test.aad, 'hex'));
 
-    var outputEncoding = test.plainIsHex ? 'hex' : 'ascii';
+    const outputEncoding = test.plainIsHex ? 'hex' : 'ascii';
 
-    var msg = decrypt.update(test.ct, 'hex', outputEncoding);
+    let msg = decrypt.update(test.ct, 'hex', outputEncoding);
     if (!test.tampered) {
       msg += decrypt.final(outputEncoding);
       assert.strictEqual(msg, test.plain);
     } else {
       // assert that final throws if input data could not be verified!
-      assert.throws(function() { decrypt.final('ascii'); }, / auth/);
+      assert.throws(function() { decrypt.final('hex'); }, errMessages.auth);
     }
-  })();
-
-  (function() {
-    if (!test.password) return;
-    if (common.hasFipsCrypto) {
-      assert.throws(() => { crypto.createCipher(test.algo, test.password); },
-                    /not supported in FIPS mode/);
-    } else {
-      var encrypt = crypto.createCipher(test.algo, test.password);
-      if (test.aad)
-        encrypt.setAAD(new Buffer(test.aad, 'hex'));
-      var hex = encrypt.update(test.plain, 'ascii', 'hex');
-      hex += encrypt.final('hex');
-      var auth_tag = encrypt.getAuthTag();
-      // only test basic encryption run if output is marked as tampered.
-      if (!test.tampered) {
-        assert.equal(hex, test.ct);
-        assert.equal(auth_tag.toString('hex'), test.tag);
-      }
-    }
-  })();
-
-  (function() {
-    if (!test.password) return;
-    if (common.hasFipsCrypto) {
-      assert.throws(() => { crypto.createDecipher(test.algo, test.password); },
-                    /not supported in FIPS mode/);
-    } else {
-      var decrypt = crypto.createDecipher(test.algo, test.password);
-      decrypt.setAuthTag(new Buffer(test.tag, 'hex'));
-      if (test.aad)
-        decrypt.setAAD(new Buffer(test.aad, 'hex'));
-      var msg = decrypt.update(test.ct, 'hex', 'ascii');
-      if (!test.tampered) {
-        msg += decrypt.final('ascii');
-        assert.equal(msg, test.plain);
-      } else {
-        // assert that final throws if input data could not be verified!
-        assert.throws(function() { decrypt.final('ascii'); }, / auth/);
-      }
-    }
-  })();
-
-  // after normal operation, test some incorrect ways of calling the API:
-  // it's most certainly enough to run these tests with one algorithm only.
-
-  if (i > 0) {
-    continue;
   }
 
-  (function() {
-    // non-authenticating mode:
-    var encrypt = crypto.createCipheriv('aes-128-cbc',
-      'ipxp9a6i1Mb4USb4', '6fKjEjR3Vl30EUYC');
-    encrypt.update('blah', 'ascii');
-    encrypt.final();
-    assert.throws(() => { encrypt.getAuthTag(); }, / state/);
-    assert.throws(() => { encrypt.setAAD(Buffer.from('123', 'ascii')); },
-                  / state/);
-  })();
+  if (test.password) {
+    if (common.hasFipsCrypto) {
+      assert.throws(() => { crypto.createCipher(test.algo, test.password); },
+                    errMessages.FIPS);
+    } else {
+      const encrypt = crypto.createCipher(test.algo, test.password);
+      if (test.aad)
+        encrypt.setAAD(Buffer.from(test.aad, 'hex'));
+      let hex = encrypt.update(test.plain, 'ascii', 'hex');
+      hex += encrypt.final('hex');
+      const auth_tag = encrypt.getAuthTag();
+      // only test basic encryption run if output is marked as tampered.
+      if (!test.tampered) {
+        assert.strictEqual(hex, test.ct);
+        assert.strictEqual(auth_tag.toString('hex'), test.tag);
+      }
+    }
+  }
 
-  (function() {
+  if (test.password) {
+    if (common.hasFipsCrypto) {
+      assert.throws(() => { crypto.createDecipher(test.algo, test.password); },
+                    errMessages.FIPS);
+    } else {
+      const decrypt = crypto.createDecipher(test.algo, test.password);
+      decrypt.setAuthTag(Buffer.from(test.tag, 'hex'));
+      if (test.aad)
+        decrypt.setAAD(Buffer.from(test.aad, 'hex'));
+      let msg = decrypt.update(test.ct, 'hex', 'ascii');
+      if (!test.tampered) {
+        msg += decrypt.final('ascii');
+        assert.strictEqual(msg, test.plain);
+      } else {
+        // assert that final throws if input data could not be verified!
+        assert.throws(function() { decrypt.final('ascii'); }, errMessages.auth);
+      }
+    }
+  }
+
+  {
     // trying to get tag before inputting all data:
-    var encrypt = crypto.createCipheriv(test.algo,
-      new Buffer(test.key, 'hex'), new Buffer(test.iv, 'hex'));
+    const encrypt = crypto.createCipheriv(test.algo,
+                                          Buffer.from(test.key, 'hex'),
+                                          Buffer.from(test.iv, 'hex'));
     encrypt.update('blah', 'ascii');
-    assert.throws(function() { encrypt.getAuthTag(); }, / state/);
-  })();
+    assert.throws(function() { encrypt.getAuthTag(); }, errMessages.state);
+  }
 
-  (function() {
+  {
     // trying to set tag on encryption object:
-    var encrypt = crypto.createCipheriv(test.algo,
-      Buffer.from(test.key, 'hex'), Buffer.from(test.iv, 'hex'));
+    const encrypt = crypto.createCipheriv(test.algo,
+                                          Buffer.from(test.key, 'hex'),
+                                          Buffer.from(test.iv, 'hex'));
     assert.throws(() => { encrypt.setAuthTag(Buffer.from(test.tag, 'hex')); },
-                  / state/);
-  })();
+                  errMessages.state);
+  }
 
-  (function() {
+  {
     // trying to read tag from decryption object:
-    var decrypt = crypto.createDecipheriv(test.algo,
-      new Buffer(test.key, 'hex'), new Buffer(test.iv, 'hex'));
-    assert.throws(function() { decrypt.getAuthTag(); }, / state/);
-  })();
+    const decrypt = crypto.createDecipheriv(test.algo,
+                                            Buffer.from(test.key, 'hex'),
+                                            Buffer.from(test.iv, 'hex'));
+    assert.throws(function() { decrypt.getAuthTag(); }, errMessages.state);
+  }
 
-  (function() {
+  {
     // trying to create cipher with incorrect IV length
     assert.throws(function() {
       crypto.createCipheriv(
@@ -449,18 +466,30 @@ for (const i in TEST_CASES) {
         Buffer.from(test.key, 'hex'),
         Buffer.alloc(0)
       );
-    }, /Invalid IV length/);
-  })();
+    }, errMessages.length);
+  }
 }
 
 // Non-authenticating mode:
-(function() {
+{
   const encrypt =
       crypto.createCipheriv('aes-128-cbc',
                             'ipxp9a6i1Mb4USb4',
                             '6fKjEjR3Vl30EUYC');
   encrypt.update('blah', 'ascii');
   encrypt.final();
-  assert.throws(() => encrypt.getAuthTag(), / state/);
-  assert.throws(() => encrypt.setAAD(Buffer.from('123', 'ascii')), / state/);
-})();
+  assert.throws(() => encrypt.getAuthTag(), errMessages.state);
+  assert.throws(() => encrypt.setAAD(Buffer.from('123', 'ascii')),
+                errMessages.state);
+}
+
+// GCM only supports specific authentication tag lengths, invalid lengths should
+// produce warnings.
+{
+  for (const length of [0, 1, 2, 4, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17]) {
+    const decrypt = crypto.createDecipheriv('aes-256-gcm',
+                                            'FxLKsqdmv0E9xrQhp0b1ZgI0K7JFZJM8',
+                                            'qkuZpJWCewa6Szih');
+    decrypt.setAuthTag(Buffer.from('1'.repeat(length)));
+  }
+}
