@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 const common = require('../common');
 const assert = require('assert');
@@ -9,7 +30,7 @@ const net = require('net');
 // and the last 10 connections are rejected.
 
 const N = 20;
-var closes = 0;
+let closes = 0;
 const waits = [];
 
 const server = net.createServer(common.mustCall(function(connection) {
@@ -26,7 +47,7 @@ server.maxConnections = N / 2;
 
 function makeConnection(index) {
   const c = net.createConnection(server.address().port);
-  var gotData = false;
+  let gotData = false;
 
   c.on('connect', function() {
     if (index + 1 < N) {
@@ -34,7 +55,7 @@ function makeConnection(index) {
     }
 
     c.on('close', function() {
-      console.error('closed %d', index);
+      console.error(`closed ${index}`);
       closes++;
 
       if (closes < N / 2) {
@@ -45,7 +66,7 @@ function makeConnection(index) {
       }
 
       if (closes === N / 2) {
-        var cb;
+        let cb;
         console.error('calling wait callback.');
         while (cb = waits.shift()) {
           cb();
@@ -76,7 +97,7 @@ function makeConnection(index) {
     if (common.isSunOS && (e.code === 'ECONNREFUSED')) {
       c.connect(server.address().port);
     }
-    console.error('error %d: %s', index, e);
+    console.error(`error ${index}: ${e}`);
   });
 }
 
