@@ -1,6 +1,27 @@
-'use strict';
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+'use strict';
 const common = require('../common');
+const fixtures = require('../common/fixtures');
 const assert = require('assert');
 
 common.globalCheck = false;
@@ -20,7 +41,7 @@ let server_tcp, server_unix, client_tcp, client_unix, replServer;
 
 
 // absolute path to test/fixtures/a.js
-const moduleFilename = require('path').join(common.fixturesDir, 'a');
+const moduleFilename = fixtures.path('a');
 
 console.error('repl test');
 
@@ -51,7 +72,7 @@ function clean_up() {
 function strict_mode_error_test() {
   send_expect([
     { client: client_unix, send: 'ref = 1',
-      expect: /^ReferenceError:\sref\sis\snot\sdefined\n\s+at\srepl:1:5/ },
+      expect: /^ReferenceError:\sref\sis\snot\sdefined\nnode via Unix socket> $/ },
   ]);
 }
 
@@ -122,7 +143,7 @@ function error_test() {
       expect: prompt_unix },
     // But passing the same string to eval() should throw
     { client: client_unix, send: 'eval("function test_func() {")',
-      expect: /\bSyntaxError: Unexpected end of input/ },
+      expect: /^SyntaxError: Unexpected end of input/ },
     // Can handle multiline template literals
     { client: client_unix, send: '`io.js',
       expect: prompt_multiline },
@@ -147,26 +168,26 @@ function error_test() {
       expect: '0.2' },
     // Can parse valid JSON
     { client: client_unix, send: 'JSON.parse(\'{"valid": "json"}\');',
-      expect: '{ valid: \'json\' }'},
+      expect: '{ valid: \'json\' }' },
     // invalid input to JSON.parse error is special case of syntax error,
     // should throw
     { client: client_unix, send: 'JSON.parse(\'{invalid: \\\'json\\\'}\');',
-      expect: /\bSyntaxError: Unexpected token i/ },
+      expect: /^SyntaxError: Unexpected token i/ },
     // end of input to JSON.parse error is special case of syntax error,
     // should throw
     { client: client_unix, send: 'JSON.parse(\'066\');',
-      expect: /\bSyntaxError: Unexpected number/ },
+      expect: /^SyntaxError: Unexpected number/ },
     // should throw
     { client: client_unix, send: 'JSON.parse(\'{\');',
-      expect: /\bSyntaxError: Unexpected end of JSON input/ },
+      expect: /^SyntaxError: Unexpected end of JSON input/ },
     // invalid RegExps are a special case of syntax error,
     // should throw
     { client: client_unix, send: '/(/;',
-      expect: /\bSyntaxError: Invalid regular expression:/ },
+      expect: /^SyntaxError: Invalid regular expression:/ },
     // invalid RegExp modifiers are a special case of syntax error,
     // should throw (GH-4012)
     { client: client_unix, send: 'new RegExp("foo", "wrong modifier");',
-      expect: /\bSyntaxError: Invalid flags supplied to RegExp constructor/ },
+      expect: /^SyntaxError: Invalid flags supplied to RegExp constructor/ },
     // strict mode syntax errors should be caught (GH-5178)
     { client: client_unix,
       send: '(function() { "use strict"; return 0755; })()',
@@ -174,7 +195,8 @@ function error_test() {
     {
       client: client_unix,
       send: '(function(a, a, b) { "use strict"; return a + b + c; })()',
-      expect: /\bSyntaxError: Duplicate parameter name not allowed in this context/ // eslint-disable-line max-len
+      expect:
+        /\bSyntaxError: Duplicate parameter name not allowed in this context/
     },
     {
       client: client_unix,
@@ -184,7 +206,8 @@ function error_test() {
     {
       client: client_unix,
       send: '(function() { "use strict"; var x; delete x; })()',
-      expect: /\bSyntaxError: Delete of an unqualified identifier in strict mode/ // eslint-disable-line max-len
+      expect:
+        /\bSyntaxError: Delete of an unqualified identifier in strict mode/
     },
     { client: client_unix,
       send: '(function() { "use strict"; eval = 17; })()',
@@ -192,7 +215,8 @@ function error_test() {
     {
       client: client_unix,
       send: '(function() { "use strict"; if (true) function f() { } })()',
-      expect: /\bSyntaxError: In strict mode code, functions can only be declared at top level or inside a block./ // eslint-disable-line max-len
+      expect:
+        /\bSyntaxError: In strict mode code, functions can only be declared at top level or inside a block\./
     },
     // Named functions can be used:
     { client: client_unix, send: 'function blah() { return 1; }',
@@ -200,7 +224,7 @@ function error_test() {
     { client: client_unix, send: 'blah()',
       expect: `1\n${prompt_unix}` },
     // Functions should not evaluate twice (#2773)
-    { client: client_unix, send: 'var I = [1,2,3,() => {}]; I.pop()',
+    { client: client_unix, send: 'var I = [1,2,3,function() {}]; I.pop()',
       expect: '[Function]' },
     // Multiline object
     { client: client_unix, send: '{ a: ',
@@ -388,6 +412,13 @@ function error_test() {
       client: client_unix, send: '(function() {\nif (false) {} /bar"/;\n}())',
       expect: `${prompt_multiline}${prompt_multiline}undefined\n${prompt_unix}`
     },
+
+    // Newline within template string maintains whitespace.
+    { client: client_unix, send: '`foo \n`',
+      expect: `${prompt_multiline}'foo \\n'\n${prompt_unix}` },
+    // Whitespace is not evaluated.
+    { client: client_unix, send: ' \t  \n',
+      expect: prompt_unix },
     // Do not parse `...[]` as a REPL keyword
     { client: client_unix, send: '...[]\n',
       expect: `${prompt_multiline}` },
