@@ -1,15 +1,14 @@
 'use strict';
-var common = require('../common');
+const common = require('../common');
 
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
-var tls = require('tls');
-var net = require('net');
 
-var server = net.Server(common.mustCall(function(raw) {
-  var pair = tls.createSecurePair(null, true, false, false);
+const tls = require('tls');
+const net = require('net');
+
+const server = net.Server(common.mustCall(function(raw) {
+  const pair = tls.createSecurePair(null, true, false, false);
   pair.on('error', function() {});
   pair.ssl.setSNICallback(common.mustCall(function() {
     raw.destroy();
