@@ -4,7 +4,7 @@
 const common = require('../common');
 const http = require('http');
 
-const server = http.createServer(function(req, res) {
+const server = http.createServer((req, res) => {
   const buffer = Buffer.alloc(0);
   // FIXME: WTF gjslint want this?
   res.writeHead(200, { 'Content-Type': 'text/html',
@@ -12,12 +12,12 @@ const server = http.createServer(function(req, res) {
   res.end(buffer);
 });
 
-server.listen(0, common.mustCall(function() {
-  http.get({ port: this.address().port }, common.mustCall(function(res) {
+server.listen(0, common.mustCall(() => {
+  http.get({ port: server.address().port }, common.mustCall((res) => {
 
     res.on('data', common.mustNotCall());
 
-    res.on('end', function(d) {
+    res.on('end', (d) => {
       server.close();
     });
   }));
